﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.IO.Ports;
using System.Drawing;
using NationalInstruments.DAQmx;
using NationalInstruments;

namespace GTTDisplay
{
    public class Display
    {
        protected int myPort;
        public SerialPort mySP;
        protected int myWidth = 480;
        protected int myHeight = 272;

        public delegate void TouchEventDelegate(string type, int X, int Y);
        public TouchEventDelegate TouchEvent;



        public Display()
        {
            myPort = 1;
        }
        public Display(int Port)
        {
            myPort = Port;
        }

        public bool InitializeDisplay()
        {
            mySP = new SerialPort("COM"+myPort.ToString(), 115200, Parity.None, 8, StopBits.One);
            try
            {            
                mySP.Encoding = Encoding.ASCII;
                mySP.Handshake = Handshake.RequestToSend;
                mySP.Open();
                mySP.ReceivedBytesThreshold = 9;
                mySP.DataReceived += port_OnReceiveDatazz;
                return true;
            }
            catch
            {
                return false;
            }
        }

        private void port_OnReceiveDatazz(object sender,
                                   SerialDataReceivedEventArgs e)
        {
            //discard all previos touches....only process the last one
        //    if(mySP.BytesToRead > 10)
         //   {
         //       mySP.Read(new byte[mySP.BytesToRead-10],0,mySP.BytesToRead-10);
         //   }

            while (mySP.BytesToRead > 8)
            {
                int sync = 0;
                while (mySP.BytesToRead > 0 && sync != 252)
                {
                    sync = mySP.ReadByte();
                }
                if (mySP.BytesToRead < 7)
                { return; }
                //found a sync and enough bytes to process....
                sync = mySP.ReadByte();
                //sync should be 135....
                sync = mySP.ReadByte();
                //sync should be 00 
                sync = mySP.ReadByte();
                //should be or 05
                int type = mySP.ReadByte();
                //should be the type of touch event....0 = down, 1 = up, 2 = move
                String rtnStr = "";
                switch (type)
                {
                    case 0: //rtnStr = "Down";
                        break;
                    case 1: rtnStr = "Up";
                        break;
                    case 2: rtnStr = "Move";
                        break;
                    default:
                        rtnStr = "Touch";
                        break;
                }
                //and now read the X and Y points of the event...
                short X = 0;
                short Y = 0;
                byte[] temp = new byte[2];
                mySP.Read(temp, 0, 2);
                byte[] temp2 = new byte[2];
                temp2[0] = temp[1];
                temp2[1] = temp[0];
                X = BitConverter.ToInt16(temp2, 0);
                mySP.Read(temp, 0, 2);
                temp2[0] = temp[1];
                temp2[1] = temp[0];
                Y = BitConverter.ToInt16(temp2, 0);

                TouchEvent(rtnStr, X, Y);
                int bytes = mySP.BytesToRead;
                byte[] debug = new byte[bytes];
              //  mySP.Read(debug,0,bytes);
            }
        }

        public int GetBytes()
        {
            return mySP.BytesToRead;
        }
        public int ComPort
        {
            get { return myPort; }
            set { myPort = value; }
        }
        public int Width
        {
            get { return myWidth; }
            set { myWidth = value; }
        }
        public int Height
        {
            get { return myHeight; }
            set { myHeight = value; }
        }

        public void ClosePort()
        {
            mySP.Close();
        }


        public void ChangeDrawingColor(Color C)
        {
            byte[] b = new byte[5];
            b[0] = 254;
            b[1] = 99;
            b[2] = C.R;
            b[3] = C.G;
            b[4] = C.B;
            mySP.Write(b, 0, b.Length);
        }
        public void ChangeBackgroundColor(Color C)
        {
            byte[] b = new byte[5];
            b[0] = 254;
            b[1] = 86;
            b[2] = C.R;
            b[3] = C.G;
            b[4] = C.B;
            mySP.Write(b, 0, b.Length);
        }
        public void DrawPoint(Point P)
        {
            byte[] b = new byte[6];
            b[0] = 254;
            b[1] = 99;
            b[2] = BitConverter.GetBytes((short)P.X)[1];
            b[3] = BitConverter.GetBytes((short)P.X)[0];
            b[4] = BitConverter.GetBytes((short)P.Y)[1];
            b[5] = BitConverter.GetBytes((short)P.Y)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawLine(Point P1,Point P2)
        {
            byte[] b = new byte[10];
            b[0] = 254;
            b[1] = 108;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)P2.X)[1];
            b[7] = BitConverter.GetBytes((short)P2.X)[0];
            b[8] = BitConverter.GetBytes((short)P2.Y)[1];
            b[9] = BitConverter.GetBytes((short)P2.Y)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void ContinueLine(Point P1)
        {
            byte[] b = new byte[6];
            b[0] = 254;
            b[1] = 101;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawRectangle(Point P1, int Width, int Height)
        {
            byte[] b = new byte[10];
            b[0] = 254;
            b[1] = 114;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)Width)[1];
            b[7] = BitConverter.GetBytes((short)Width)[0];
            b[8] = BitConverter.GetBytes((short)Height)[1];
            b[9] = BitConverter.GetBytes((short)Height)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawFilledRectangle(Point P1, int Width, int Height)
        {
            byte[] b = new byte[10];
            b[0] = 254;
            b[1] = 120;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)Width)[1];
            b[7] = BitConverter.GetBytes((short)Width)[0];
            b[8] = BitConverter.GetBytes((short)Height)[1];
            b[9] = BitConverter.GetBytes((short)Height)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawCircle(Point P1, int Radius)
        {
            byte[] b = new byte[8];
            b[0] = 254;
            b[1] = 123;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)Radius)[1];
            b[7] = BitConverter.GetBytes((short)Radius)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawFilledCircle(Point P1, int Radius)
        {
            byte[] b = new byte[8];
            b[0] = 254;
            b[1] = 124;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)Radius)[1];
            b[7] = BitConverter.GetBytes((short)Radius)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawEllipse(Point P1, int XRadius,int YRadius)
        {
            byte[] b = new byte[10];
            b[0] = 254;
            b[1] = 125;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)XRadius)[1];
            b[7] = BitConverter.GetBytes((short)XRadius)[0];
            b[8] = BitConverter.GetBytes((short)YRadius)[1];
            b[9] = BitConverter.GetBytes((short)YRadius)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawFilledEllipse(Point P1, int XRadius, int YRadius)
        {
            byte[] b = new byte[10];
            b[0] = 254;
            b[1] = 126;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)XRadius)[1];
            b[7] = BitConverter.GetBytes((short)XRadius)[0];
            b[8] = BitConverter.GetBytes((short)YRadius)[1];
            b[9] = BitConverter.GetBytes((short)YRadius)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawRoundedRectangle(Point P1, int Width, int Height,int Radius)
        {
            byte[] b = new byte[12];
            b[0] = 254;
            b[1] = 127;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)Width)[1];
            b[7] = BitConverter.GetBytes((short)Width)[0];
            b[8] = BitConverter.GetBytes((short)Height)[1];
            b[9] = BitConverter.GetBytes((short)Height)[0];
            b[10] = BitConverter.GetBytes((short)Radius)[1];
            b[11] = BitConverter.GetBytes((short)Radius)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawFilledRoundedRectangle(Point P1, int Width, int Height, int Radius)
        {
            byte[] b = new byte[12];
            b[0] = 254;
            b[1] = 128;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)Width)[1];
            b[7] = BitConverter.GetBytes((short)Width)[0];
            b[8] = BitConverter.GetBytes((short)Height)[1];
            b[9] = BitConverter.GetBytes((short)Height)[0];
            b[10] = BitConverter.GetBytes((short)Radius)[1];
            b[11] = BitConverter.GetBytes((short)Radius)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void DrawTriangle(Point P1, Point P2, Point P3)
        {
            byte[] b = new byte[14];
            b[0] = 254;
            b[1] = 129;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)P2.X)[1];
            b[7] = BitConverter.GetBytes((short)P2.X)[0];
            b[8] = BitConverter.GetBytes((short)P2.Y)[1];
            b[9] = BitConverter.GetBytes((short)P2.Y)[0];
            b[10] = BitConverter.GetBytes((short)P3.X)[1];
            b[11] = BitConverter.GetBytes((short)P3.X)[0];
            b[12] = BitConverter.GetBytes((short)P3.Y)[1];
            b[13] = BitConverter.GetBytes((short)P3.Y)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void ClearScreen()
        {
            byte[] b = new byte[2];
            b[0] = 254;
            b[1] = 88;
            mySP.Write(b, 0, b.Length);
        }
        public void CreateBarGraph(int Index,int Min, int Max,int X,int Y,int Width,int Height, Color Foreground, Color Background, int Orientation)
        {
            byte[] b = new byte[22];
            b[0] = 254;
            b[1] = 103;
            b[2] = (byte)Index;
            b[3] = BitConverter.GetBytes((short)Min)[1];
            b[4] = BitConverter.GetBytes((short)Min)[0];
            b[5] = BitConverter.GetBytes((short)Max)[1];
            b[6] = BitConverter.GetBytes((short)Max)[0];
            b[7] = BitConverter.GetBytes((short)X)[1];
            b[8] = BitConverter.GetBytes((short)X)[0];
            b[9] = BitConverter.GetBytes((short)Y)[1];
            b[10] = BitConverter.GetBytes((short)Y)[0];
            b[11] = BitConverter.GetBytes((short)Width)[1];
            b[12] = BitConverter.GetBytes((short)Width)[0];
            b[13] = BitConverter.GetBytes((short)Height)[1];
            b[14] = BitConverter.GetBytes((short)Height)[0];
            b[15] = Foreground.R;
            b[16] = Foreground.G;
            b[17] = Foreground.B;
            b[18] = Background.R;
            b[19] = Background.G;
            b[20] = Background.B;
            b[21] = (byte)Orientation;
            mySP.Write(b, 0, b.Length);
        }
        public void UpdateSingleBargraph(int Index, int Value)
        {
            byte[] b = new byte[5];
            b[0] = 254;
            b[1] = 105;
            b[2] = (byte)(Index);
            b[3] = BitConverter.GetBytes((short)Value)[1];
            b[4] = BitConverter.GetBytes((short)Value)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void UpdateMultipleBargraphs(int StartIndex, int[] Values)
        {
            byte[] b = new byte[4+(Values.Length*2)];
            b[0] = 254;
            b[1] = 106;
            b[2] = (byte)(StartIndex);
            b[3] = (byte)(Values.Length);
            int i = 4;
            foreach (int Value in Values)
            {
                b[i++] = BitConverter.GetBytes((short)Value)[1];
                b[i++] = BitConverter.GetBytes((short)Value)[0];
            }
            mySP.Write(b, 0, b.Length);
        }
        public void EnableManualUpdate(bool enable)
        {
            byte[] b = new byte[3];
            b[0] = 254;
            b[1] = 90;
            b[2] =(byte)(enable ? 1 : 0);
            mySP.Write(b, 0, b.Length);
        }
        public void ManualUpdate()
        {
            byte[] b = new byte[2];
            b[0] = 254;
            b[1] = 91;
            mySP.Write(b, 0, b.Length);
        }
        public void SetBacklightBrightness(short Brightness)
        {
            byte[] b = new byte[3];
            b[0] = 254;
            b[1] = 153;
            b[2] = (byte)Brightness;
            mySP.Write(b, 0, b.Length);
        }
        public void SetTextColor(Color C)
        {
            byte[] b = new byte[5];
            b[0] = 254;
            b[1] = 46;
            b[2] = C.R;
            b[3] = C.G;
            b[4] = C.B;
            mySP.Write(b, 0, b.Length);
        }
        public void SetTextWindow(Point P1, int Width, int Height)
        {
            byte[] b = new byte[10];
            b[0] = 254;
            b[1] = 43;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            b[6] = BitConverter.GetBytes((short)Width)[1];
            b[7] = BitConverter.GetBytes((short)Width)[0];
            b[8] = BitConverter.GetBytes((short)Height)[1];
            b[9] = BitConverter.GetBytes((short)Height)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void SetTextInsertionPoint(Point P1)
        {
            byte[] b = new byte[6];
            b[0] = 254;
            b[1] = 121;
            b[2] = BitConverter.GetBytes((short)P1.X)[1];
            b[3] = BitConverter.GetBytes((short)P1.X)[0];
            b[4] = BitConverter.GetBytes((short)P1.Y)[1];
            b[5] = BitConverter.GetBytes((short)P1.Y)[0];
            mySP.Write(b, 0, b.Length);
        }
        public void WriteText(String S)
        {
            mySP.Write(S);
        }
        public void SetScrollMode(int Mode)
        {
            byte[] b = new byte[6];
            b[0] = 254;
            b[1] = 81;
            b[2] = (byte)Mode;
            mySP.Write(b, 0, b.Length);
        }
        public void SetupTouchRegion(int Index,Point P1, int Width, int Height, int UpBuffer, int DownBuffer)
        {
            byte[] b = new byte[13];
            b[0] = 254;
            b[1] = 132;
            b[2] = (byte)Index;
            b[3] = BitConverter.GetBytes((short)P1.X)[1];
            b[4] = BitConverter.GetBytes((short)P1.X)[0];
            b[5] = BitConverter.GetBytes((short)P1.Y)[1];
            b[6] = BitConverter.GetBytes((short)P1.Y)[0];
            b[7] = BitConverter.GetBytes((short)Width)[1];
            b[8] = BitConverter.GetBytes((short)Width)[0];
            b[9] = BitConverter.GetBytes((short)Height)[1];
            b[10] = BitConverter.GetBytes((short)Height)[0];
            b[11] = (byte)UpBuffer;
            b[12] = (byte)DownBuffer;
            mySP.Write(b, 0, b.Length);
        }
        public void ClearTouchRegion(int Index)
        {
            byte[] b = new byte[3];
            b[0] = 254;
            b[1] = 133;
            b[2] = (byte)Index;
            mySP.Write(b, 0, b.Length);
        }
        public void ClearAllTouchRegions()
        {
            byte[] b = new byte[2];
            b[0] = 254;
            b[1] = 134;
            mySP.Write(b, 0, b.Length);
        }
        public void ChangeTouchReportingStyle(int Style)
        {
            byte[] b = new byte[3];
            b[0] = 254;
            b[1] = 135;
            b[2] = (byte)Style;
            mySP.Write(b, 0, b.Length);
        }
        public void LoadBMPToBuffer(int Index,String filename)
        {
            byte[] b = new byte[3];
            b[0] = 254;
            b[1] = 95;
            b[2] = (byte)Index;
            mySP.Write(b, 0, b.Length);
            WriteText(filename);
        }
        public void LoadNineSliceToBuffer(int Index, String filename)
        {
            byte[] b = new byte[3];
            b[0] = 254;
            b[1] = 144;
            b[2] = (byte)Index;
            mySP.Write(b, 0, b.Length);
            WriteText(filename);
        }


    }
}
